# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@free.fr)
# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu

set rcsId {$Id: stagraph.tcl,v 1.1.1.2 1999/11/20 20:51:29 jfontain Exp $}


class dataStackedGraph {

    proc dataStackedGraph {this parentPath args} composite {
        [new barchart $parentPath\
            -title {} -barmode stack -topmargin 3 -bufferelements 0 -plotborderwidth 1 -plotbackground black -barwidth 0\
        ] $args
    } blt2DViewer {$widget::($this,path)} bltGraph {$widget::($this,path)} {
        $widget::($this,path) grid off                                                                      ;# grid is on by default
        composite::complete $this
    }

    proc ~dataStackedGraph {this} {
        if {[string length $composite::($this,-deletecommand)]>0} {
            uplevel #0 $composite::($this,-deletecommand)                                   ;# always invoke command at global level
        }
    }

    proc iconData {} {
        return {
            R0lGODdhKAAoAKUAAHh4eMjMyAAAANDU0Pj8+DBgIDBgGHi8WKDscKj0eKD0eJjkcJDUaJjkaHBIAPDcSODEOMikKDg4OFAIYBBgeGgYeIgQoFCUqKjU4IC8
            yLgo0KAYuLAYyLAg0Ng4+KgYyNhA+OBI+LgY2OBQ+NAY8MgY6OBY+OBg+Ngo+Ohw+AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
            AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAAKAAoAAAG/kCAcEgsGo/IYQAgaDqf0Kh02lwKAthsYIDlbrVer3Y8ZgYI6DQaoG6z2+n3Gm09wwlyeN6t
            flsHd3iBgoF5fmZDBYpJAIoGBUaPj0Z1aE0HBwWEagUIngiaaQUJCQoLoYcCgASXB6B5ip4LCKaKiqaeDKh0ZpYCmAygop/ECAvHyAzKuwR1Qq3BDbayC8rW
            1gjXmU5Cf76YBw3V1g3X5ueZQ7xX3+Dn7++YkELrq60HDODA8Obg86lLmPzSR3CftYL+uAGoxGogQoL53AHzp64ZE3sODzjYuHHiQ4L/1tlptfGBSZMOPiIM
            afHKs18OTEI4ibIgx5T6Cihk2MRB/oSZMiEA7agx5kmi6ei1xBghQlChQFEaBTo0JUueAppC3UpVJs2vDq5efNn0J9ezaE9CcCChSbdeDbWaRUt3awQHFbGW
            hTCX68++QsvezQu3iWC+Zv8KXrwYr1IrUBgjRsy48l0oViZobkKBguWmnTszDk3BQYXTp61YWN3kwgXPlSlgmI0BNmjaGW5q0GBlA4cOrS/UXtyZNm3SxjEI
            4OjBg5UPHD4EF25btvEMyXFjyLB8IwgQzz8AF+D6ePHZ2DOo375+O/vuDkKEeK5BxPTs7NXr37//AvzvVogg4HSu8WfggRm4JsBqFowwQoAiaEBgggVWeAGF
            FF7omoIMQjoYIAklELjhiCSWOOKCq5lgghUghkieiTDG6B+DJ5zAYgkniCjjjihaUKMVKKCQwpBEFmnkkUgiuQQZTDbppJNBAAA7
        }
    }

    proc options {this} {
        set samples [expr {$global::graphNumberOfIntervals+1}]
        # force size and interval values
        return [list\
            [list -deletecommand {} {}]\
            [list -draggable draggable Draggable 0 0]\
            [list -height height Height $global::viewerHeight]\
            [list -interval interval Interval 5]\
            [list -samples samples Samples $samples $samples]\
            [list -width width Width $global::viewerWidth]\
        ]
    }

    proc set-deletecommand {this value} {}

    foreach option {-height -width} {
        proc set$option {this value} "\$widget::(\$this,path) configure $option \$value"
    }

    proc set-draggable {this value} {
        if {$composite::($this,complete)} {
            error {option -draggable cannot be set dynamically}
        }
        if {$value} {
            blt2DViewer::allowDrag $this
        }
    }

    proc set-interval {this value} {
        bltGraph::setRange $this [expr {($composite::($this,-samples)-1)*$value}]
        bltGraph::updateGraduations $this
    }

    proc set-samples {this value} {                                                                     ;# stored at composite level
        if {$composite::($this,complete)} {
            error {option -samples cannot be set dynamically}
        }
        bltGraph::setRange $this [expr {($value-1)*$composite::($this,-interval)}]
    }

    proc updateTimeDisplay {this seconds} {
        bltGraph::xAxisUpdate $this $seconds
    }

    proc newElement {this path args} {                                                          ;# invoked from 2D viewer base class
        return [eval new element $path $args]
    }

    proc updateElement {this element seconds value} {                           ;# value is either a valid number or the ? character
        element::update $element $seconds $value
    }

}


class dataStackedGraph {

    class element {

        proc element {this path args} switched {$args} {
            variable x$this
            variable y$this

            blt::vector create x${this}(0)                                                                     ;# x axis data vector
            blt::vector create y${this}(0)                                                                     ;# y axis data vector
            # use object identifier as element identifier, and handle void values
            $path element create $this -label {} -xdata x$this -ydata y$this
            set ($this,path) $path
            switched::complete $this
        }

        proc ~element {this} {
            variable x$this
            variable y$this

            blt::vector destroy x$this y$this
            [set ($this,path)] element delete $this
            if {[string length $switched::($this,-deletecommand)]>0} {
                uplevel #0 $switched::($this,-deletecommand)                                ;# always invoke command at global level
            }
        }

        proc options {this} {
            return [list\
                [list -color black black]\
                [list -deletecommand {} {}]\
                [list -label {} {}]\
            ]
        }

        proc set-color {this value} {
            [set ($this,path)] element configure $this -foreground $value
        }

        proc set-label {this value} {
            [set ($this,path)] element configure $this -label $value
        }

        proc set-deletecommand {this value} {}                                                   ;# data is stored at switched level

        proc update {this x y} {                                                    ;# y is either a valid number or the ? character
            variable x$this
            variable y$this

            set path [set ($this,path)]

            set minimum [$path axis cget x -min]
            set length [llength [x$this search 0 $minimum]]
            incr length -2
            catch {                                                   ;# scroll by removing points to the left of the x axis minimum
                x$this delete :$length
                y$this delete :$length
            }
            set void [expr {[string compare $y ?]==0}]                                                                 ;# void value
            if {[x$this length]>0} {                                                                            ;# after first point
                blt::vector create add(0)
                # create a vector with one value per half pixel (to avoid holes due to precision errors) between last and new value
                add seq [set x${this}(end)] $x [expr {([$path axis cget x -max]-$minimum)/[$path extents plotwidth]/2}]
                add delete 0                                                                                    ;# remove last value
                if {[add length]<=1} {                                             ;# there was no room for some intermediate points
                    blt::vector destroy add
                }
            }
            if {[info exists add]} {                                              ;# append a group of points to make a filled curve
                x$this append add
                blt::vector create limits(0)
                if {$void} {
                    limits append 0 0
                } else {
                    limits append [set y${this}(end)] $y
                }
                limits populate add [expr {[add length]-1}]
                add delete 0                                                                                    ;# remove last value
                y$this append add
                blt::vector destroy add limits
            } else {                                                                                        ;# append a single point
                x$this append $x                                                                                 ;# append new value
                if {$void} {
                    y$this append 0
                } else {
                    y$this append $y
                }
            }
            [set ($this,path)] element configure $this -label "$switched::($this,-label): $y"
        }

    }

}
